package controllers

import (
	"encoding/json"
	"eta_gn/eta_index_lib/logic"
	"eta_gn/eta_index_lib/models"
	"eta_gn/eta_index_lib/services"
	"eta_gn/eta_index_lib/utils"
	"fmt"
	"strconv"
	"time"
)

// GnController
type GnController struct {
	BaseAuthController
}

// Refresh
// @Title 国能物产-刷新指标接口
// @Description 刷新指标接口
// @Success 200 {object} models.RefreshEdbInfoReq
// @router /refresh [post]
func (this *GnController) Refresh() {
	br := new(models.BaseResponse).Init()
	var cacheKey string
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req models.RefreshEdbInfoReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.EdbCode == "" {
		br.Msg = "请输入指标编码！"
		br.ErrMsg = "请输入指标编码，指标编码为空"
		return
	}
	if req.EdbInfoId <= 0 {
		br.Msg = "请输入指标ID！"
		br.ErrMsg = "请输入指标ID"
		return
	}
	if req.Source <= 0 {
		br.Msg = "请输入指标来源"
		br.ErrMsg = "指标来源为空"
		return
	}
	startDate := req.StartDate
	if startDate == "" {
		startDate = utils.BASE_START_DATE
	}

	// 加锁
	cacheKey = utils.CACHE_EDB_DATA_REFRESH + strconv.Itoa(req.Source) + "_" + req.EdbCode
	if utils.Rc.IsExist(cacheKey) {
		br.Ret = 501
		br.Success = true
		br.Msg = "系统处理中，请稍后重试"
		return
	}
	utils.Rc.SetNX(cacheKey, 1, 1*time.Minute)
	defer func() {
		_ = utils.Rc.Delete(cacheKey)
	}()

	// 校验指标和数据源
	edbInfo, e := models.GetEdbInfoById(req.EdbInfoId)
	if e != nil {
		br.Msg = "指标信息有误"
		br.ErrMsg = "查询指标信息失败, Err: " + e.Error()
		return
	}
	sourceItem := models.GetEdbSourceBySourceId(req.Source)
	if sourceItem == nil {
		br.Msg = "数据源有误"
		br.ErrMsg = fmt.Sprintf("数据源有误, Source: %d", req.Source)
		return
	}

	// 从桥接服务获取指标和数据
	var params models.BridgeGnIndexDataParams
	params.IndexCode = edbInfo.OriginalEdbCode
	params.StartDate = startDate
	params.EndDate = utils.BASE_END_DATE
	indexDataList, e := services.GetGnIndexDataListFromBridge(params)
	if e != nil {
		br.Msg = "获取指标失败"
		br.ErrMsg = "获取指标数据失败, Err: " + e.Error()
		return
	}

	// 刷新指标数据
	e = models.RefreshEdbDataFromGn(req.Source, edbInfo.SubSource, req.EdbInfoId, sourceItem.TableName1, req.EdbCode, startDate, indexDataList)
	if e != nil {
		br.Msg = "刷新失败"
		br.ErrMsg = "刷新国能指标失败, Err: " + e.Error()
		return
	}

	// 更新指标最大最小值
	e, errMsg := models.UnifiedModifyEdbInfoMaxAndMinInfo(edbInfo)
	if e != nil {
		br.Msg = errMsg
		br.ErrMsg = e.Error()
		return
	}

	// 更新ES
	go logic.UpdateEs(edbInfo.EdbInfoId)

	br.Ret = 200
	br.Success = true
	br.Msg = "操作成功"
}
