package models

import (
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"fmt"

	"strings"
	"time"
)

// EdbDataTradeAnalysis 持仓分析指标数据
type EdbDataTradeAnalysis struct {
	EdbDataId     int       `gorm:"primaryKey;autoIncrement;column:edb_data_id"`
	EdbInfoId     int       `gorm:"column:edb_info_id" description:"指标ID"`
	EdbCode       string    `gorm:"column:edb_code" description:"指标编码"`
	DataTime      time.Time `gorm:"column:data_time" description:"数据日期"`
	Value         float64   `gorm:"column:value" description:"数据值"`
	CreateTime    time.Time `gorm:"column:status" description:"创建时间"`
	ModifyTime    time.Time `gorm:"column:create_time" description:"修改时间"`
	DataTimestamp int64     `gorm:"column:modify_time" description:"数据日期时间戳"`
}

func (m *EdbDataTradeAnalysis) TableName() string {
	return "edb_data_trade_analysis"
}

type EdbDataTradeAnalysisCols struct {
	PrimaryId     string
	EdbInfoId     string
	EdbCode       string
	DataTime      string
	Value         string
	CreateTime    string
	ModifyTime    string
	DataTimestamp string
}

func (m *EdbDataTradeAnalysis) Cols() EdbDataTradeAnalysisCols {
	return EdbDataTradeAnalysisCols{
		PrimaryId:     "edb_data_id",
		EdbInfoId:     "edb_info_id",
		EdbCode:       "edb_code",
		DataTime:      "data_time",
		Value:         "value",
		CreateTime:    "create_time",
		ModifyTime:    "modify_time",
		DataTimestamp: "data_timestamp",
	}
}

func (m *EdbDataTradeAnalysis) Create() (err error) {

	err = global.DEFAULT_DmSQL.Create(m).Error
	return
}

func (m *EdbDataTradeAnalysis) CreateMulti(items []*EdbDataTradeAnalysis) (err error) {
	if len(items) == 0 {
		return
	}

	err = global.DEFAULT_DmSQL.CreateInBatches(items, 500).Error
	return
}

func (m *EdbDataTradeAnalysis) Update(cols []string) (err error) {

	err = global.DEFAULT_DmSQL.Model(m).Select(cols).Updates(m).Error
	return
}

func (m *EdbDataTradeAnalysis) Remove() (err error) {

	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DmSQL.Raw(sql, m.EdbDataId).Error
	return
}

func (m *EdbDataTradeAnalysis) MultiRemove(ids []int) (err error) {
	if len(ids) == 0 {
		return
	}

	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.Cols().PrimaryId, utils.GetOrmInReplace(len(ids)))
	err = global.DEFAULT_DmSQL.Raw(sql, ids).Error
	return
}

func (m *EdbDataTradeAnalysis) RemoveByCondition(condition string, pars []interface{}) (err error) {
	if condition == "" {
		return
	}

	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s`, m.TableName(), condition)
	err = global.DEFAULT_DmSQL.Exec(sql, pars...).Error
	return
}

func (m *EdbDataTradeAnalysis) GetItemById(id int) (item *EdbDataTradeAnalysis, err error) {

	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DmSQL.Raw(sql, id).First(&item).Error
	return
}

func (m *EdbDataTradeAnalysis) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *EdbDataTradeAnalysis, err error) {

	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).First(&item).Error
	return
}

func (m *EdbDataTradeAnalysis) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {

	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Scan(&count).Error
	return
}

func (m *EdbDataTradeAnalysis) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*EdbDataTradeAnalysis, err error) {

	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Scan(&items).Error
	return
}

func (m *EdbDataTradeAnalysis) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*EdbDataTradeAnalysis, err error) {

	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Scan(&items).Error
	return
}

// MultiUpdateValue 批量更新数据值
func (m *EdbDataTradeAnalysis) MultiUpdateValue(items []*EdbDataTradeAnalysis) (err error) {
	if len(items) == 0 {
		return
	}
	sql := fmt.Sprintf("UPDATE %s SET %s = ?, %s = NOW() WHERE %s = ?", m.TableName(), m.Cols().Value, m.Cols().ModifyTime, m.Cols().PrimaryId)
	for _, v := range items {
		err = global.DEFAULT_DmSQL.Exec(sql, v.Value, v.EdbDataId).Error
		if err != nil {
			return
		}
	}
	return
}
