package models

import (
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"time"
)

// EdbInfoCalculateMapping 计算指标于基础指标，关系表
type EdbInfoCalculateMapping struct {
	EdbInfoCalculateMappingId int       `gorm:"primaryKey;autoIncrement;column:edb_info_calculate_mapping_id"`
	EdbInfoId                 int       `gorm:"column:edb_info_id" description:"计算指标id"`
	Source                    int       `gorm:"column:source" description:"计算指标来源"`
	SourceName                string    `gorm:"column:source_name" description:"计算指标来源名称"`
	EdbCode                   string    `gorm:"column:edb_code" description:"计算指标编码"`
	FromEdbInfoId             int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode               string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName               string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource                int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName            string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	MoveValue                 int       `gorm:"column:move_value" description:"移动的值，小于0是提前，0是不变，大于0是滞后"`
	Sort                      int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime                time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                time.Time `gorm:"column:modify_time" description:"修改时间"`
	FromTag                   string    `gorm:"column:from_tag" description:"来源指标标签"`
	FromSubSource             int       `gorm:"column:from_sub_source" description:"渠道子数据库来源"`
}

// EdbInfoCalculateMappingInfo
// @Description: 计算指标与基础指标关系表
type EdbInfoCalculateMappingInfo struct {
	EdbInfoCalculateMappingId int       `gorm:"primaryKey;autoIncrement;column:edb_info_calculate_mapping_id"`
	EdbInfoId                 int       `gorm:"column:edb_info_id" description:"计算指标id"`
	Source                    int       `gorm:"column:source" description:"计算指标来源"`
	SourceName                string    `gorm:"column:source_name" description:"计算指标来源名称"`
	EdbCode                   string    `gorm:"column:edb_code" description:"计算指标编码"`
	FromEdbInfoId             int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode               string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName               string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource                int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName            string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	MoveValue                 int       `gorm:"column:move_value" description:"移动的值，小于0是提前，0是不变，大于0是滞后"`
	Sort                      int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime                time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                time.Time `gorm:"column:modify_time" description:"修改时间"`
	FromTag                   string    `gorm:"column:from_tag" description:"来源指标标签"`
	FromEdbType               int       `description:"来源指标类型:1:基础指标,2:计算指标"`
	FromEdbInfoType           int       `description:"来源指标类型: 0-基础指标; 1-预测指标"`
	FromClassifyId            int       `description:"来源指标分类ID"`
	FromUniqueCode            string    `description:"来源指标唯一编码"`
	NoUpdate                  int8      `description:"是否停止更新，0：继续更新；1：停止更新"`
}

// AddEdbInfoCalculateMappingMulti 批量添加指标关系表
func AddEdbInfoCalculateMappingMulti(items []*EdbInfoCalculateMapping) (err error) {
	err = global.DEFAULT_DmSQL.CreateInBatches(items, 500).Error
	return
}

// AddEdbInfoCalculateMappingMulti 批量添加指标关系表
func (e *EdbInfoCalculateMapping) Add() (err error) {
	err = global.DEFAULT_DmSQL.Create(e).Error
	return
}

// EdbInfoCalculateMappingView
type EdbInfoCalculateMappingView struct {
	EdbInfoCalculateMappingId int       `gorm:"primaryKey;autoIncrement;column:edb_info_calculate_mapping_id"`
	EdbInfoId                 int       `gorm:"column:edb_info_id" description:"计算指标id"`
	Source                    int       `gorm:"column:source" description:"计算指标来源"`
	SourceName                string    `gorm:"column:source_name" description:"计算指标来源名称"`
	EdbCode                   string    `gorm:"column:edb_code" description:"计算指标编码"`
	FromEdbInfoId             int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode               string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName               string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource                int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName            string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	Sort                      int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime                time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                time.Time `gorm:"column:modify_time" description:"修改时间"`
	FromTag                   string    `gorm:"column:from_tag" description:"来源指标标签"`
	StartDate                 string    `description:"开始日期"`
	EndDate                   string    `description:"结束日期"`
	CalculateFormula          string    `description:"N值"`
	MoveType                  int       `description:"移动方式:1:领先(默认),2:滞后"`
	MoveFrequency             string    `description:"移动频度"`
}

// GetEdbInfoCalculateMappingDetail 获取单条关联指标
func GetEdbInfoCalculateMappingDetail(edbInfoId int) (item *EdbInfoCalculateMappingView, err error) {
	sql := ` SELECT a.*,b.start_date,b.end_date,b.calculate_formula,b.move_type,b.move_frequency FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoId).First(&item).Error
	return
}

// EdbInfoCalculateDetail
type EdbInfoCalculateDetail struct {
	EdbInfoCalculateId int       `gorm:"primaryKey;autoIncrement;column:edb_info_calculate_mapping_id"`
	EdbInfoId          int       `gorm:"column:edb_info_id" description:"计算指标id"`
	EdbCode            string    `gorm:"column:edb_code" description:"计算指标编码"`
	FromEdbInfoId      int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode        string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName        string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource         int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName     string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	Sort               int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime         time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime         time.Time `gorm:"column:modify_time" description:"修改时间"`
	FromTag            string    `gorm:"column:from_tag" description:"来源指标标签"`
	StartDate          string    `description:"开始日期"`
	EndDate            string    `description:"结束日期"`
	EdbType            int       `description:"指标类型:1:基础指标,2:计算指标"`
}

// GetEdbInfoCalculateDetailList 获取关联指标列表
func GetEdbInfoCalculateDetailList(edbInfoId int) (list []*EdbInfoCalculateDetail, err error) {
	sql := ` SELECT a.*,b.start_date,b.end_date,b.edb_type FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? ORDER BY sort ASC `

	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoId).Scan(&list).Error

	return
}

// GetEdbInfoByEdbCode 根据指标code获取指标信息
func GetEdbInfoCalculateByEdbCode(edbCode, fromEdbCode string) (item *EdbInfoCalculateMapping, err error) {
	sql := ` SELECT * FROM edb_info_calculate_mapping WHERE edb_code=? AND from_edb_code=? `
	err = global.DEFAULT_DmSQL.Raw(sql, edbCode, fromEdbCode).First(&item).Error

	return
}

// GetEdbInfoCalculateMappingListByEdbInfoIds 根据生成的指标id获取来源的指标id列表
func GetEdbInfoCalculateMappingListByEdbInfoIds(edbInfoIds []int) (items []*EdbInfoCalculateMappingInfo, err error) {
	sql := ` SELECT a.*,b.edb_type as from_edb_type,b.edb_info_type as from_edb_info_type, b.unique_code AS from_unique_code, b.classify_id AS from_classify_id,b.no_update FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id in (` + utils.GetOrmInReplace(len(edbInfoIds)) + `) `
	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoIds).Scan(&items).Error

	return
}

// GetEdbInfoCalculateMappingListByEdbInfoId 根据生成的指标id获取来源的指标id列表
func GetEdbInfoCalculateMappingListByEdbInfoId(edbInfoId int) (items []*EdbInfoCalculateMappingInfo, err error) {
	sql := ` SELECT a.*,b.edb_type as from_edb_type,b.edb_info_type as from_edb_info_type, b.unique_code AS from_unique_code, b.classify_id AS from_classify_id,b.no_update FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoId).Scan(&items).Error

	return
}
